@@ -7,7 +7,7 @@ test() {
 	od -t x1 $1.bin | sed 's/  */ /g' | sed -e '$a\' >$1.r
 	cat $1.r $1.out $1.err >$1.obj.output 2>/dev/null
 	rm $1.bin $1.r $1.out $1.err 2>/dev/null
-	diff -Z -b $1.obj.output $1.obj.answer
+	diff -b $1.obj.output $1.obj.answer
 	if [ $? -eq 0 ]; then
 		rm $1.obj.output
 	fi
@@ -21,7 +21,7 @@ testlink() {
 	od -t x1 $1.bin | sed 's/  */ /g' | sed -e '$a\' >$1.r
 	cat $1.r $1.out $1.err >$1.bin.output 2>/dev/null
 	rm $.bin $1.r $1.out $1.err 2>/dev/null
-	diff -Z -b $1.bin.output $1.bin.answer
+	diff -b $1.bin.output $1.bin.answer
 	if [ $? -eq 0 ]; then
 		rm $1.bin.output
 	fi
