@@ -82,7 +82,7 @@ static bool
 handle_ASW(uint8_t baseOpcode, SAddressingMode* addrMode) {
 	if (addrMode->mode == MODE_ABS) {
 		sect_OutputConst8(baseOpcode);
-		sect_OutputExpr16(addrMode->expr);
+		x65_OutputU16Expression(addrMode->expr);
 	} else {
 		assert(false);
 	}
@@ -200,14 +200,46 @@ handle_LongBranch(uint8_t baseOpcode, SAddressingMode* addrMode) {
 static bool
 handle_LDQImm(SAddressingMode* addrMode) {
 	if (opt_Current->machineOptions->synthesized) {
-		sect_OutputConst8(0xA9);	// LDA
-		sect_OutputExpr8(expr_And(expr_Copy(addrMode->expr), expr_Const(0xFF)));
-		sect_OutputConst8(0xA2);	// LDX
-		sect_OutputExpr8(expr_And(expr_Asr(expr_Copy(addrMode->expr), expr_Const(8)), expr_Const(0xFF)));
-		sect_OutputConst8(0xA0);	// LDY
-		sect_OutputExpr8(expr_And(expr_Asr(expr_Copy(addrMode->expr), expr_Const(16)), expr_Const(0xFF)));
-		sect_OutputConst8(0xA3);	// LDZ
-		sect_OutputExpr8(expr_And(expr_Asr(expr_Copy(addrMode->expr), expr_Const(24)), expr_Const(0xFF)));
+		if (expr_IsConstant(addrMode->expr)) {
+			uint32_t imm = (uint32_t) addrMode->expr->value.integer;
+			uint8_t a = (uint8_t) (imm & 0xFF);
+			uint8_t x = (uint8_t) ((imm >> 8) & 0xFF);
+			uint8_t y = (uint8_t) ((imm >> 16) & 0xFF);
+			uint8_t z = (uint8_t) ((imm >> 24) & 0xFF);
+
+			sect_OutputConst8(0xA9);	// LDA
+			sect_OutputConst8(a);
+
+			if (a == x) {
+				sect_OutputConst8(0xAA);	// TAX
+			} else {
+				sect_OutputConst8(0xA2);	// LDX
+				sect_OutputConst8(x);
+			}
+
+			if (a == y) {
+				sect_OutputConst8(0xA8);	// TAX
+			} else {
+				sect_OutputConst8(0xA0);	// LDY
+				sect_OutputConst8(y);
+			}
+
+			if (a == z) {
+				sect_OutputConst8(0x4B);	// TAZ
+			} else {
+				sect_OutputConst8(0xA3);	// LDZ
+				sect_OutputConst8(z);
+			}
+		} else {
+			sect_OutputConst8(0xA9);	// LDA
+			sect_OutputExpr8(expr_And(expr_Copy(addrMode->expr), expr_Const(0xFF)));
+			sect_OutputConst8(0xA2);	// LDX
+			sect_OutputExpr8(expr_And(expr_Asr(expr_Copy(addrMode->expr), expr_Const(8)), expr_Const(0xFF)));
+			sect_OutputConst8(0xA0);	// LDY
+			sect_OutputExpr8(expr_And(expr_Asr(expr_Copy(addrMode->expr), expr_Const(16)), expr_Const(0xFF)));
+			sect_OutputConst8(0xA3);	// LDZ
+			sect_OutputExpr8(expr_And(expr_Asr(expr_Copy(addrMode->expr), expr_Const(24)), expr_Const(0xFF)));
+		}
 	} else {
 		err_Error(MERROR_SYNTHESIZED);
 	}
@@ -256,18 +288,18 @@ static SQMnemonic
 g_qMnemonics[T_45GS02_STQ + 1 - T_45GS02_ADCQ] = {
 	{ MODE_ZP | MODE_ABS | MODE_IND_ZP | MODE_45GS02_IND_ZP_QUAD, T_6502_ADC },
 	{ MODE_ZP | MODE_ABS | MODE_IND_ZP | MODE_45GS02_IND_ZP_QUAD, T_6502_AND },
-	{ MODE_45GS02_Q | MODE_ZP | MODE_ABS | MODE_ZP_X | MODE_ABS_X, T_6502_ASL },
-	{ MODE_45GS02_Q | MODE_ZP | MODE_ZP_X, T_4510_ASR },
+	{ MODE_IMM | MODE_45GS02_Q | MODE_ZP | MODE_ABS | MODE_ZP_X | MODE_ABS_X, T_6502_ASL },
+	{ MODE_IMM | MODE_45GS02_Q | MODE_ZP | MODE_ZP_X, T_4510_ASR },
 	{ MODE_ZP | MODE_ABS, T_6502_BIT },
 	{ MODE_ZP | MODE_ABS | MODE_IND_ZP | MODE_45GS02_IND_ZP_QUAD, T_6502_CMP },
 	{ MODE_45GS02_Q | MODE_ZP | MODE_ABS | MODE_ZP_X | MODE_ABS_X, T_6502_DEC },
 	{ MODE_ZP | MODE_ABS | MODE_IND_ZP | MODE_45GS02_IND_ZP_QUAD, T_6502_EOR },
 	{ MODE_45GS02_Q | MODE_ZP | MODE_ABS | MODE_ZP_X | MODE_ABS_X, T_6502_INC },
 	{ MODE_IMM | MODE_ZP | MODE_ABS | MODE_4510_IND_ZP_Z | MODE_45GS02_IND_ZP_Z_QUAD, T_6502_LDA },
-	{ MODE_45GS02_Q | MODE_ZP | MODE_ABS | MODE_ZP_X | MODE_ABS_X, T_6502_LSR },
+	{ MODE_IMM |MODE_45GS02_Q | MODE_ZP | MODE_ABS | MODE_ZP_X | MODE_ABS_X, T_6502_LSR },
 	{ MODE_ZP | MODE_ABS | MODE_IND_ZP | MODE_45GS02_IND_ZP_QUAD, T_6502_ORA },
-	{ MODE_45GS02_Q | MODE_ZP | MODE_ABS | MODE_ZP_X | MODE_ABS_X, T_6502_ROL },
-	{ MODE_45GS02_Q | MODE_ZP | MODE_ABS | MODE_ZP_X | MODE_ABS_X, T_6502_ROR },
+	{ MODE_IMM | MODE_45GS02_Q | MODE_ZP | MODE_ABS | MODE_ZP_X | MODE_ABS_X, T_6502_ROL },
+	{ MODE_IMM | MODE_45GS02_Q | MODE_ZP | MODE_ABS | MODE_ZP_X | MODE_ABS_X, T_6502_ROR },
 	{ MODE_ZP | MODE_ABS | MODE_IND_ZP | MODE_45GS02_IND_ZP_QUAD, T_6502_SBC },
 	{ MODE_ZP | MODE_ABS | MODE_IND_ZP | MODE_45GS02_IND_ZP_QUAD, T_6502_STA }
 };
@@ -291,12 +323,26 @@ x65_Handle4510Instruction(ETargetToken token, uint32_t allowedModes) {
 	} else if (T_45GS02_ADCQ <= token && token <= T_45GS02_STQ) {
 		if (opt_Current->machineOptions->cpu & MOPT_CPU_45GS02) {
 			SQMnemonic* handler = &g_qMnemonics[token - T_45GS02_ADCQ];
+			allowedModes = allowedModes & handler->allowedModes;
 
 			if (handler->token >= T_4510_ASR && handler->token <= T_4510_TZA) {
 				SAddressingMode addrMode;
 				SParser* handler4510 = &g_instructionHandlers[handler->token - T_4510_ASR];
 
 				if (x65_ParseAddressingMode(&addrMode, allowedModes, handler4510->immSize) && (addrMode.mode & allowedModes)) {
+					// ASRQ #imm
+					if ((handler->token == T_4510_ASR) && addrMode.mode == MODE_IMM && expr_IsConstant(addrMode.expr) && opt_Current->machineOptions->synthesized) {
+						SAddressingMode modeImm1 = { .mode = MODE_45GS02_Q };
+
+						for (uint32_t i = 0; i < (uint32_t) addrMode.expr->value.integer; ++i) {
+							sect_OutputConst8(0x42);
+							sect_OutputConst8(0x42);
+							if (!handler4510->handler(handler4510->baseOpcode, &modeImm1))
+								return false;
+						}
+						return true;
+					}
+
 					sect_OutputConst8(0x42);
 					sect_OutputConst8(0x42);
 					return handler4510->handler(handler4510->baseOpcode, &addrMode);
@@ -320,6 +366,18 @@ x65_Handle4510Instruction(ETargetToken token, uint32_t allowedModes) {
 							break;
 					}
 
+					if ((handler->token == T_6502_ASL || handler->token == T_6502_LSR || handler->token == T_6502_ROL || handler->token == T_6502_ROR) && addrMode.mode == MODE_IMM && expr_IsConstant(addrMode.expr) && opt_Current->machineOptions->synthesized) {
+						// ASLQ, LSRQ, ROLQ, RORQ #imm
+						SAddressingMode modeImmA = { .mode = MODE_A };
+						for (uint32_t i = 0; i < (uint32_t) addrMode.expr->value.integer; ++i) {
+							sect_OutputConst8(0x42);
+							sect_OutputConst8(0x42);
+							if (!x65_HandleTokenAddressMode(handler->token, &modeImmA))
+								return false;
+						}
+						return true;
+					} 
+
 					if (token == T_45GS02_LDQ && addrMode.mode == MODE_IMM) {
 						return handle_LDQImm(&addrMode);
 					} 
