@@ -47,7 +47,7 @@ typedef enum {
     ERROR_SYMBOL_EXPORT,
     ERROR_SECTION_DATA,
     ERROR_IMPORT_DEFINED,
-    ERROR_SECTION_FULL,
+    ERROR_unused_3,
     ERROR_SECTION_MISSING,
     ERROR_EXPR_CONST_RELOC,
     ERROR_EXPR_TWO_POWER,
