@@ -16,28 +16,27 @@
     along with ASMotor.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <memory.h>
 #include <string.h>
 #include <assert.h>
 
-#include "util.h"
-#include "file.h"
+#include "lists.h"
 #include "mem.h"
+#include "str.h"
+#include "util.h"
 
-#include "xasm.h"
 #include "dependency.h"
 #include "errors.h"
 #include "expression.h"
-#include "lexer_context.h"
-#include "includes.h"
 #include "linemap.h"
 #include "options.h"
-#include "parse.h"
 #include "patch.h"
 #include "section.h"
 #include "symbol.h"
+#include "xasm.h"
 
 
 #define SECTION_GROWTH_AMOUNT 0x4000U
@@ -91,7 +90,6 @@ createSection(const string* name) {
 	memset(newSection, 0, sizeof(SSection));
 
 	newSection->name = str_Copy(name);
-	newSection->freeSpace = xasm_Configuration->maxSectionSize;
 
 	if (sect_Sections != NULL) {
 		SSection* lastSection = sect_Sections;
@@ -153,15 +151,10 @@ checkAvailableSpace(uint32_t count) {
 	assert((uint32_t) xasm_Configuration->minimumWordSize <= count);
 
 	if (sect_Current != NULL) {
-		if (count <= sect_Current->freeSpace) {
-			if (currentSectionType() == GROUP_TEXT) {
-				growCurrentSection(count);
-			}
-			return true;
-		} else {
-            err_Error(ERROR_SECTION_FULL);
-			return false;
+		if (currentSectionType() == GROUP_TEXT) {
+			growCurrentSection(count);
 		}
+		return true;
 	} else {
         err_Error(ERROR_SECTION_MISSING);
 		return false;
@@ -225,7 +218,6 @@ sect_OutputConst8(uint8_t value) {
 			case GROUP_TEXT: {
 				linemap_AddCurrent();
 
-				sect_Current->freeSpace -= 1;
 				sect_Current->data[sect_Current->usedSpace++] = value;
 				sect_Current->cpuProgramCounter += 1;
 				break;
@@ -252,9 +244,10 @@ sect_OutputReloc8(SExpression* expression) {
 				linemap_AddCurrent();
 
 				patch_Create(sect_Current, sect_Current->usedSpace, expression, PATCH_8);
+
+				sect_Current->data[sect_Current->usedSpace++] = 0;
+
 				sect_Current->cpuProgramCounter += 1;
-				sect_Current->usedSpace += 1;
-				sect_Current->freeSpace -= 1;
 				break;
 			}
 			case GROUP_BSS: {
@@ -308,7 +301,6 @@ sect_OutputConst16(uint16_t value) {
 						break;
 					}
 				}
-				sect_Current->freeSpace -= 2;
 				sect_Current->cpuProgramCounter += 2 / xasm_Configuration->minimumWordSize;
 				break;
 			}
@@ -335,8 +327,10 @@ sect_OutputReloc16(SExpression* expression) {
 
 				patch_Create(sect_Current, sect_Current->usedSpace, expression,
 							 opt_Current->endianness == ASM_LITTLE_ENDIAN ? PATCH_LE_16 : PATCH_BE_16);
-				sect_Current->freeSpace -= 2;
-				sect_Current->usedSpace += 2;
+
+				sect_Current->data[sect_Current->usedSpace++] = 0;
+				sect_Current->data[sect_Current->usedSpace++] = 0;
+
 				sect_Current->cpuProgramCounter += 2 / xasm_Configuration->minimumWordSize;
 				break;
 			}
@@ -396,7 +390,6 @@ sect_OutputConst32(uint32_t value) {
 						break;
 					}
 				}
-				sect_Current->freeSpace -= 4;
 				sect_Current->cpuProgramCounter += 4 / xasm_Configuration->minimumWordSize;
 				break;
 			}
@@ -423,9 +416,13 @@ sect_OutputRel32(SExpression* expression) {
 
 				patch_Create(sect_Current, sect_Current->usedSpace, expression,
 							 opt_Current->endianness == ASM_LITTLE_ENDIAN ? PATCH_LE_32 : PATCH_BE_32);
-				sect_Current->freeSpace -= 4;
+
+				sect_Current->data[sect_Current->usedSpace++] = 0;
+				sect_Current->data[sect_Current->usedSpace++] = 0;
+				sect_Current->data[sect_Current->usedSpace++] = 0;
+				sect_Current->data[sect_Current->usedSpace++] = 0;
+
 				sect_Current->cpuProgramCounter += 4 / xasm_Configuration->minimumWordSize;
-				sect_Current->usedSpace += 4;
 				break;
 			}
 			case GROUP_BSS: {
@@ -494,7 +491,6 @@ sect_OutputBinaryFile(string* filename) {
 					linemap_AddCurrent();
 
 					size_t read = fread(&sect_Current->data[sect_Current->usedSpace], sizeof(uint8_t), size, fileHandle);
-					sect_Current->freeSpace -= size;
 					sect_Current->usedSpace += size;
 					sect_Current->cpuProgramCounter += size / xasm_Configuration->minimumWordSize;
 					if (read != size) {
@@ -551,7 +547,6 @@ sect_SkipBytes(uint32_t count) {
 				break;
 			}
 			case GROUP_BSS: {
-				sect_Current->freeSpace -= count;
 				sect_Current->usedSpace += count;
 				sect_Current->cpuProgramCounter += count / xasm_Configuration->minimumWordSize;
 				break;
