@@ -22,9 +22,9 @@
 #include <string.h>
 #include <assert.h>
 
-#include "util.h"
 #include "mem.h"
 
+#include "parse_symbol.h"
 #include "xasm.h"
 #include "symbol.h"
 #include "lexer_context.h"
@@ -364,6 +364,9 @@ sym_CreateStructure(string* name, int32_t initialCount) {
 extern void
 sym_EndStructure(void) {
 	if (sym_CurrentScope != NULL && sym_CurrentScope->type == SYM_STRUCTURE) {
+		if (g_rsSymbol != NULL)
+			sym_CurrentScope->value.integer = g_rsSymbol->value.integer;
+		
 		sym_CurrentScope = NULL;
 	} else {
 		err_Error(ERROR_NOT_IN_STRUCTURE_SCOPE);
@@ -382,14 +385,14 @@ sym_CreateLabel(string* name) {
 			if (!isLocalName(name))
 				sym_CurrentScope = symbol;
 
-			if ((sect_Current->flags & (SECTF_LOADFIXED | SECTF_ORGFIXED)) == 0) {
-				SET_TYPE_AND_FLAGS(symbol, SYM_LABEL);
-				symbol->section = sect_Current;
-				symbol->value.integer = sect_Current->cpuProgramCounter;
-			} else {
+			if ((sect_Current->flags & (SECTF_LOADFIXED | SECTF_ORGFIXED)) != 0 && (sect_Current->flags & SECTF_BANKFIXED) == 0) {
 				SET_TYPE_AND_FLAGS(symbol, SYM_EQU);
 				symbol->value.integer = sect_Current->cpuProgramCounter + sect_Current->cpuAdjust
 									  + sect_Current->cpuOrigin;
+			} else {
+				SET_TYPE_AND_FLAGS(symbol, SYM_LABEL);
+				symbol->section = sect_Current;
+				symbol->value.integer = sect_Current->cpuProgramCounter;
 			}
 			return symbol;
 		} else {
