@@ -16,12 +16,13 @@
     along with ASMotor.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include <stdbool.h>
+#include <stdint.h>
+
 #include "mem.h"
-#include "str.h"
 #include "types.h"
 
 #include "group.h"
-#include "symbol.h"
 #include "xlink.h"
 
 #include <string.h>
@@ -111,17 +112,17 @@ group_InitCommonGameboy(void) {
     //	Create VRAM group
 
     group = group_Create("VRAM", 1);
-    group->pools[0] = pool_Create(-1, 0x8000, 0, 0x2000, false);
+    group->pools[0] = pool_Create(-1, UINT32_MAX, 0x8000, 0, 0x2000, false);
 
     //	Create BSS group
 
     group = group_Create("BSS", 1);
-    group->pools[0] = pool_Create(-1, 0xC000, 0, 0x2000, false);
+    group->pools[0] = pool_Create(-1, UINT32_MAX, 0xC000, 0, 0x2000, false);
 
     //	Create HRAM group
 
     group = group_Create("HRAM", 1);
-    group->pools[0] = pool_Create(-1, 0xFF80, 0, 0x7F, false);
+    group->pools[0] = pool_Create(-1, UINT32_MAX, 0xFF80, 0, 0x7F, false);
 }
 
 static MemoryGroup*
@@ -137,7 +138,7 @@ group_FindByName(const char* name) {
 
 static bool
 group_AllocateMemoryFromGroup(MemoryGroup* group, uint32_t size, int32_t bankId, int32_t* cpuByteLocation,
-                              int32_t* cpuBank, int32_t* imageLocation) {
+                              int32_t* cpuBank, int32_t* imageLocation, uint32_t* overlay) {
     for (int32_t i = 0; i < group->totalPools; ++i) {
         MemoryPool* pool = group->pools[i];
 
@@ -145,6 +146,7 @@ group_AllocateMemoryFromGroup(MemoryGroup* group, uint32_t size, int32_t bankId,
             if (pool_AllocateMemory(pool, size, cpuByteLocation)) {
                 *cpuBank = pool->cpuBank;
                 *imageLocation = pool->imageLocation == -1 ? -1 : pool->imageLocation + *cpuByteLocation - (int32_t) pool->cpuByteLocation;
+				*overlay = pool->overlay;
                 return true;
             }
         }
@@ -154,7 +156,7 @@ group_AllocateMemoryFromGroup(MemoryGroup* group, uint32_t size, int32_t bankId,
 
 static bool
 group_AllocateAbsoluteFromGroup(MemoryGroup* group, uint32_t size, int32_t bankId, int32_t cpuByteLocation,
-                                int32_t* cpuBank, int32_t* imageLocation) {
+                                int32_t* cpuBank, int32_t* imageLocation, uint32_t* overlay) {
     for (int32_t i = 0; i < group->totalPools; ++i) {
         MemoryPool* pool = group->pools[i];
 
@@ -162,6 +164,7 @@ group_AllocateAbsoluteFromGroup(MemoryGroup* group, uint32_t size, int32_t bankI
             if (pool_AllocateAbsolute(pool, size, cpuByteLocation)) {
                 *cpuBank = pool->cpuBank;
                 *imageLocation = pool->imageLocation == -1 ? -1 : pool->imageLocation + cpuByteLocation - (int32_t) pool->cpuByteLocation;
+				*overlay = pool->overlay;
                 return true;
             }
         }
@@ -171,7 +174,7 @@ group_AllocateAbsoluteFromGroup(MemoryGroup* group, uint32_t size, int32_t bankI
 
 static bool
 group_AllocateAlignedFromGroup(MemoryGroup* group, uint32_t size, int32_t bankId, int32_t byteAlign,
-                               int32_t* cpuByteLocation, int32_t* cpuBank, int32_t* imageLocation) {
+                               int32_t* cpuByteLocation, int32_t* cpuBank, int32_t* imageLocation, uint32_t* overlay) {
     for (int32_t i = 0; i < group->totalPools; ++i) {
         MemoryPool* pool = group->pools[i];
 
@@ -179,6 +182,7 @@ group_AllocateAlignedFromGroup(MemoryGroup* group, uint32_t size, int32_t bankId
             if (pool_AllocateAligned(pool, size, byteAlign, cpuByteLocation)) {
                 *cpuBank = pool->cpuBank;
                 *imageLocation = pool->imageLocation == -1 ? -1 : pool->imageLocation + *cpuByteLocation - (int32_t) pool->cpuByteLocation;
+				*overlay = pool->overlay;
                 return true;
             }
         }
@@ -220,10 +224,11 @@ group_Create(const char* groupName, uint32_t totalBanks) {
 
 
 extern MemoryPool*
-pool_Create(int32_t imageLocation, uint32_t cpuByteLocation, int32_t cpuBank, uint32_t size, bool onlyAbs) {
+pool_Create(int32_t imageLocation, uint32_t overlay, uint32_t cpuByteLocation, int32_t cpuBank, uint32_t size, bool onlyAbs) {
     MemoryPool* pool = (MemoryPool*) mem_Alloc(sizeof(MemoryPool));
 
     pool->imageLocation = imageLocation;
+	pool->overlay = overlay;
     pool->cpuByteLocation = cpuByteLocation;
     pool->cpuBank = cpuBank;
     pool->size = size;
@@ -250,24 +255,23 @@ pool_Free(MemoryPool* pool) {
 
 bool
 group_AllocateMemory(const char* groupName, uint32_t size, int32_t bankId, int32_t* cpuByteLocation, int32_t* cpuBank,
-                     int32_t* imageLocation) {
+                     int32_t* imageLocation, uint32_t* overlay) {
     MemoryGroup* group = group_FindByName(groupName);
-    return group_AllocateMemoryFromGroup(group, size, bankId, cpuByteLocation, cpuBank, imageLocation);
-
+    return group_AllocateMemoryFromGroup(group, size, bankId, cpuByteLocation, cpuBank, imageLocation, overlay);
 }
 
 bool
 group_AllocateAbsolute(const char* groupName, uint32_t size, int32_t bankId, int32_t cpuByteLocation, int32_t* cpuBank,
-                       int32_t* imageLocation) {
+                       int32_t* imageLocation, uint32_t* overlay) {
     MemoryGroup* group = group_FindByName(groupName);
-    return group_AllocateAbsoluteFromGroup(group, size, bankId, cpuByteLocation, cpuBank, imageLocation);
+    return group_AllocateAbsoluteFromGroup(group, size, bankId, cpuByteLocation, cpuBank, imageLocation, overlay);
 }
 
 bool
 group_AllocateAligned(const char* groupName, uint32_t size, int32_t bankId, int32_t byteAlign, int32_t* cpuByteLocation,
-                      int32_t* cpuBank, int32_t* imageLocation) {
+                      int32_t* cpuBank, int32_t* imageLocation, uint32_t* overlay) {
     MemoryGroup* group = group_FindByName(groupName);
-    return group_AllocateAlignedFromGroup(group, size, bankId, byteAlign, cpuByteLocation, cpuBank, imageLocation);
+    return group_AllocateAlignedFromGroup(group, size, bankId, byteAlign, cpuByteLocation, cpuBank, imageLocation, overlay);
 }
 
 void
@@ -276,7 +280,7 @@ group_SetupGameboy(void) {
     MemoryGroup* group;
 
     for (int i = 0; i < 256; ++i)
-        codepools[i] = pool_Create(i * 0x4000, i == 0 ? 0x0000 : 0x4000, i, 0x4000, false);
+        codepools[i] = pool_Create(i * 0x4000, UINT32_MAX, i == 0 ? 0x0000 : 0x4000, i, 0x4000, false);
 
     //	Create HOME group
 
@@ -305,7 +309,7 @@ group_SetupSmallGameboy(void) {
     MemoryPool* codepool;
     MemoryGroup* group;
 
-    codepool = pool_Create(0, 0x0000, 0, 0x8000, false);
+    codepool = pool_Create(0, UINT32_MAX, 0x0000, 0, 0x8000, false);
 
     //	Create HOME group
 
@@ -333,8 +337,8 @@ group_SetupAmiga(void) {
     MemoryPool* codepool;
     MemoryPool* chippool;
 
-    codepool = pool_Create(0x00200000, 0, 0, 0x3FFFFFFF, false);
-    chippool = pool_Create(0x00000000, 0, 0, 0x001FFFFF, false);
+    codepool = pool_Create(0x00200000, UINT32_MAX, 0, 0, 0x3FFFFFFF, false);
+    chippool = pool_Create(0x00000000, UINT32_MAX, 0, 0, 0x001FFFFF, false);
 
     //	Create CODE group
     group = group_Create("CODE", 1);
@@ -354,18 +358,18 @@ group_SetupAmiga(void) {
 
     //	Create BSS group
     group = group_Create("BSS", 1);
-    group->pools[0] = pool_Create(-1, 0, 0, 0x3FFFFFFF, false);
+    group->pools[0] = pool_Create(-1, UINT32_MAX, 0, 0, 0x3FFFFFFF, false);
 
     //	Create BSS_C group
     group = group_Create("BSS_C", 1);
-    group->pools[0] = pool_Create(-1, 0, 0, 0x3FFFFFFF, false);
+    group->pools[0] = pool_Create(-1, UINT32_MAX, 0, 0, 0x3FFFFFFF, false);
 }
 
 void
 group_SetupSegaMegaDrive(void) {
     MemoryGroup* group;
-    MemoryPool* rompool = pool_Create(0, 0x000000, 0, 0x400000, false);
-    MemoryPool* rampool = pool_Create(0, 0xFF0000, 0, 0x10000, false);
+    MemoryPool* rompool = pool_Create(0, UINT32_MAX, 0x000000, 0, 0x400000, false);
+    MemoryPool* rampool = pool_Create(0, UINT32_MAX, 0xFF0000, 0, 0x10000, false);
 
     //	Create CODE group
     group = group_Create("CODE", 1);
@@ -384,8 +388,8 @@ void
 group_SetupSegaMasterSystem(int size) {
     MemoryGroup* group;
 
-    MemoryPool* homepool = pool_Create(0, 0, 0, 0x400, false);
-    MemoryPool* codepool = pool_Create(0x400, 0x400, 0, size - 16 - 0x400, false);
+    MemoryPool* homepool = pool_Create(0, UINT32_MAX, 0, 0, 0x400, false);
+    MemoryPool* codepool = pool_Create(0x400, UINT32_MAX, 0x400, 0, size - 16 - 0x400, false);
 
     //	Create CODE group
 
@@ -407,7 +411,7 @@ group_SetupSegaMasterSystem(int size) {
     //	Create BSS group
 
     group = group_Create("BSS", 1);
-    group->pools[0] = pool_Create(-1, 0xC000, 0, 0x1FF8, false);
+    group->pools[0] = pool_Create(-1, UINT32_MAX, 0xC000, 0, 0x1FF8, false);
 }
 
 void
@@ -416,12 +420,12 @@ group_SetupSegaMasterSystemBanked(void) {
     MemoryPool* codepools[65];
     MemoryGroup* group;
 
-    homepool = pool_Create(0, 0, 0, 0x400, false);
+    homepool = pool_Create(0, UINT32_MAX, 0, 0, 0x400, false);
 
-    codepools[0] = pool_Create(0x400, 0x400, 0, 0x4000 - 0x400, false);
-    codepools[1] = pool_Create(0x4000, 0x4000, 1, 0x3FF0, false);
+    codepools[0] = pool_Create(0x400, UINT32_MAX, 0x400, 0, 0x4000 - 0x400, false);
+    codepools[1] = pool_Create(0x4000, UINT32_MAX, 0x4000, 1, 0x3FF0, false);
     for (int i = 2; i <= 63; ++i)
-        codepools[i] = pool_Create(i * 0x4000, 0x8000, i, 0x4000, false);
+        codepools[i] = pool_Create(i * 0x4000, UINT32_MAX, 0x8000, i, 0x4000, false);
 
     //	Create HOME group
 
@@ -445,7 +449,7 @@ group_SetupSegaMasterSystemBanked(void) {
     //	Create BSS group
 
     group = group_Create("BSS", 1);
-    group->pools[0] = pool_Create(-1, 0xC000, 0, 0x1FF8, false);
+    group->pools[0] = pool_Create(-1, UINT32_MAX, 0xC000, 0, 0x1FF8, false);
 }
 
 
@@ -453,8 +457,8 @@ void
 group_SetupHC8XXROM(void) {
     MemoryGroup* group;
 
-    MemoryPool* codepool = pool_Create(0, 0, 1, 0x4000, false);
-	MemoryPool* bss = pool_Create(-1, 0x0000, 0x80, 0x4000, false);
+    MemoryPool* codepool = pool_Create(0, UINT32_MAX, 0, 1, 0x4000, false);
+	MemoryPool* bss = pool_Create(-1, UINT32_MAX, 0x0000, 0x80, 0x4000, false);
 
     //	Create HOME group
 
@@ -503,8 +507,8 @@ group_SetupHC8XXSmall(void) {
 	/* HC800, CODE: 64 KiB text + data + bss */
     MemoryGroup* group;
 
-    MemoryPool* home = pool_Create(0, 0, 0x81, 0x4000, false);
-    MemoryPool* shared = pool_Create(0, 0x4000, 0x82, 0xC000, false);
+    MemoryPool* home = pool_Create(0, UINT32_MAX, 0, 0x81, 0x4000, false);
+    MemoryPool* shared = pool_Create(0, UINT32_MAX, 0x4000, 0x82, 0xC000, false);
 
     //	Create HOME group
 
@@ -551,10 +555,10 @@ group_SetupHC8XXSmallHarvard(void) {
 	/* HC800 CODE: 64 KiB text, DATA: 64 KiB data + bss */
     MemoryGroup* group;
 
-    MemoryPool* home = pool_Create(0, 0, 0x81, 0x4000, false);
-    MemoryPool* codeShared = pool_Create(0, 0x4000, 0x82, 0xC000, false);
-    MemoryPool* data = pool_Create(0, 0, 0x85, 0x4000, false);
-    MemoryPool* dataShared = pool_Create(0, 0x4000, 0x86, 0xC000, false);
+    MemoryPool* home = pool_Create(0, UINT32_MAX, 0, 0x81, 0x4000, false);
+    MemoryPool* codeShared = pool_Create(0, UINT32_MAX, 0x4000, 0x82, 0xC000, false);
+    MemoryPool* data = pool_Create(0, UINT32_MAX, 0, 0x85, 0x4000, false);
+    MemoryPool* dataShared = pool_Create(0, UINT32_MAX, 0x4000, 0x86, 0xC000, false);
 
     //	Create HOME group
 
@@ -601,13 +605,13 @@ group_SetupHC8XXMedium(void) {
 	/* HC800, CODE: 32 KiB text + data + bss, CODE: 32 KiB sized banks text */
     MemoryGroup* group;
 
-    MemoryPool* home = pool_Create(0, 0, 0x81, 0x4000, false);
-    MemoryPool* shared = pool_Create(0, 0x4000, 0x82, 0x8000, false);
+    MemoryPool* home = pool_Create(0, UINT32_MAX, 0, 0x81, 0x4000, false);
+    MemoryPool* shared = pool_Create(0, UINT32_MAX, 0x4000, 0x82, 0x8000, false);
 	int firstCodeBank = 0x83;
 	int codeBanks = (0x100 - firstCodeBank) / 2;
 	MemoryPool* banks[HC800_MAX_BANKS];
 	for (int i = 0; i < codeBanks; ++i) {
-		banks[i] = pool_Create(0, 0x8000, i * 2 + firstCodeBank, 0x8000, false);
+		banks[i] = pool_Create(0, UINT32_MAX, 0x8000, i * 2 + firstCodeBank, 0x8000, false);
 	}
 
     //	Create HOME group
@@ -659,16 +663,16 @@ group_SetupHC8XXMediumHarvard(void) {
 	/* HC800, CODE: 32 KiB text, CODE: 32 KiB sized text banks, DATA: 64 KiB data + bss */
     MemoryGroup* group;
 
-    MemoryPool* home = pool_Create(0, 0, 0x81, 0x4000, false);
-    MemoryPool* shared = pool_Create(0, 0x4000, 0x82, 0x4000, false);
-    MemoryPool* data = pool_Create(0, 0, 0x83, 0x4000, false);
-    MemoryPool* dataShared = pool_Create(0, 0x4000, 0x84, 0xC000, false);
+    MemoryPool* home = pool_Create(0, UINT32_MAX, 0, 0x81, 0x4000, false);
+    MemoryPool* shared = pool_Create(0, UINT32_MAX, 0x4000, 0x82, 0x4000, false);
+    MemoryPool* data = pool_Create(0, 0, UINT32_MAX, 0x83, 0x4000, false);
+    MemoryPool* dataShared = pool_Create(0, UINT32_MAX, 0x4000, 0x84, 0xC000, false);
 
 	int firstCodeBank = 0x87;
 	int codeBanks = (0x100 - firstCodeBank) / 2;
 	MemoryPool* banks[HC800_MAX_BANKS];
 	for (int i = 0; i < codeBanks; ++i) {
-		banks[i] = pool_Create(0, 0x8000, i * 2 + firstCodeBank, 0x8000, false);
+		banks[i] = pool_Create(0, UINT32_MAX, 0x8000, i * 2 + firstCodeBank, 0x8000, false);
 	}
 
     //	Create HOME group
@@ -721,13 +725,13 @@ group_SetupHC8XXLarge(void) {
 
     MemoryGroup* group;
 
-    MemoryPool* home = pool_Create(0, 0, 0x81, 0x4000, false);
-    MemoryPool* shared = pool_Create(0, 0x4000, 0x82, 0x4000, false);
+    MemoryPool* home = pool_Create(0, UINT32_MAX, 0, 0x81, 0x4000, false);
+    MemoryPool* shared = pool_Create(0, UINT32_MAX, 0x4000, 0x82, 0x4000, false);
 	int firstCodeBank = 0x83;
 	int codeBanks = (0x100 - firstCodeBank) / 2;
 	MemoryPool* banks[HC800_MAX_BANKS];
 	for (int i = 0; i < codeBanks; ++i) {
-		banks[i] = pool_Create(0, 0x8000, i * 2 + firstCodeBank, 0x8000, false);
+		banks[i] = pool_Create(0, UINT32_MAX, 0x8000, i * 2 + firstCodeBank, 0x8000, false);
 	}
 
     //	Create HOME group
@@ -786,10 +790,10 @@ group_SetupHC8XXLargeHarvard(void) {
 
     MemoryGroup* group;
 
-    MemoryPool* home = pool_Create(0, 0, 0x81, 0x4000, false);
-    MemoryPool* sharedCode = pool_Create(0, 0x4000, 0x82, 0x4000, false);
-	MemoryPool* bssPool = pool_Create(0, 0x0000, 0x83, 0x4000, false);
-	MemoryPool* sharedBssPool = pool_Create(0, 0x4000, 0x84, 0xC000, false);
+    MemoryPool* home = pool_Create(0, UINT32_MAX, 0, 0x81, 0x4000, false);
+    MemoryPool* sharedCode = pool_Create(0, UINT32_MAX, 0x4000, 0x82, 0x4000, false);
+	MemoryPool* bssPool = pool_Create(0, UINT32_MAX, 0x0000, 0x83, 0x4000, false);
+	MemoryPool* sharedBssPool = pool_Create(0, UINT32_MAX, 0x4000, 0x84, 0xC000, false);
 	int firstCodeBank = 0x87;
 	int codeBanks = (0x100 - firstCodeBank) / 2;
 
@@ -805,7 +809,7 @@ group_SetupHC8XXLargeHarvard(void) {
     group->pools[0] = home;
     group->pools[1] = sharedCode;
 	for (int i = 0; i < codeBanks; ++i) {
-		group->pools[i + 2] = pool_Create(0, 0x8000, firstCodeBank + i * 2, 0x8000, false);
+		group->pools[i + 2] = pool_Create(0, UINT32_MAX, 0x8000, firstCodeBank + i * 2, 0x8000, false);
 	}
 
     //	Create DATA group
@@ -841,7 +845,7 @@ void
 group_SetupCoCo(void) {
     MemoryGroup* group;
 
-    MemoryPool* system_ram = pool_Create(0, 0x600, 0, 0x8000 - 0x600, false);
+    MemoryPool* system_ram = pool_Create(0, UINT32_MAX, 0x600, 0, 0x8000 - 0x600, false);
 
     //	Create CODE group
 
@@ -860,3 +864,14 @@ group_SetupCoCo(void) {
 }
 
 
+extern bool
+group_NeedsOverlay(void) {
+	for (MemoryGroup* group = s_machineGroups; group != NULL; group = group->nextGroup) {
+		for (int32_t i = 0; i < group->totalPools; ++i) {
+			if (group->pools[i]->overlay != UINT32_MAX)
+				return true;
+		}
+	}
+
+	return false;
+}
