@@ -17,10 +17,11 @@
 */
 
 #include <ctype.h>
-#include <stdarg.h>
+#include <stdbool.h>
+#include <stdint.h>
+#include <stdlib.h>
+#include <stdio.h>
 
-#include "util.h"
-#include "file.h"
 #include "str.h"
 
 #include "amiga.h"
@@ -32,8 +33,9 @@
 #include "group.h"
 #include "hc800.h"
 #include "image.h"
+#include "listfile.h"
 #include "mapfile.h"
-#include "memorymap.h"
+#include "machinedefinition.h"
 #include "object.h"
 #include "patch.h"
 #include "section.h"
@@ -62,6 +64,7 @@ static int g_binaryPad = -1;
 static const char* g_smartlink = NULL;
 static const char* g_entry = NULL;
 static const char* g_mapFilename = NULL;
+static const char* g_listFilename = NULL;
 static bool g_targetDefined = false;
 
 const char* g_outputFilename = NULL;
@@ -71,6 +74,11 @@ format_SupportsReloc(FileFormat type) {
     return type == FILE_FORMAT_AMIGA_EXECUTABLE || type == FILE_FORMAT_AMIGA_LINK_OBJECT;
 }
 
+static bool
+format_SupportsOverlay(FileFormat type) {
+    return type == FILE_FORMAT_CBM_PRG || type == FILE_FORMAT_MEGA65_PRG;
+}
+
 static bool
 format_SupportsOnlySectionRelativeReloc(FileFormat type) {
     return type == FILE_FORMAT_AMIGA_EXECUTABLE;
@@ -106,14 +114,14 @@ printUsage(void) {
            "          -csmd       Sega Mega Drive\n"
            "          -csms       Sega Master System (32 KiB)\n"
            "          -csmsb      Sega Master System banked (64+ KiB)\n"
-           "          -chc800b    HC800 16 KiB ROM\n"
-           "          -chc800s    HC800 small mode (64 KiB text + data + bss)\n"
-           "          -chc800sh   HC800 small Harvard mode (64 KiB text + data, 64 KiB data + bss)\n"
-           "          -chc800m    HC800 medium mode (32 KiB text + data + bss, 32 KiB sized\n"
+           "          -chc8b      HC800 16 KiB ROM\n"
+           "          -chc8s      HC800 small mode (64 KiB text + data + bss)\n"
+           "          -chc8sh     HC800 small Harvard mode (64 KiB text + data, 64 KiB data + bss)\n"
+           "          -chc8m      HC800 medium mode (32 KiB text + data + bss, 32 KiB sized\n"
 		   "                      banks text)\n"
-           "          -chc800mh   HC800 medium Harvard executable (32 KiB text + 32 KiB\n"
+           "          -chc8mh     HC800 medium Harvard executable (32 KiB text + 32 KiB\n"
 		   "                      sized text banks, 64 KiB data + bss)\n"
-           "          -chc800l    HC800 large mode (32 KiB text + data + bss, 32 KiB sized\n"
+           "          -chc8l      HC800 large mode (32 KiB text + data + bss, 32 KiB sized\n"
 		   "                      banks text + data + bss)\n"
            "          -cfxa2560x  Foenix A2560X/K\n"
            "          -cfxf256jrs Foenix F256 Jr. small mode (64 KiB, I/O hole)\n"
@@ -139,12 +147,14 @@ printUsage(void) {
            "          -fcocobin   TRS-80 Color Computer .bin\n"
 		   "          -fmega65    MEGA65 .PRG\n"
 		   "\n"
-           "    -m<mapfile> Write a mapfile to <mapfile>\n"
+           "    -l<listfile> Write a listfile to <listfile>\n"
 		   "\n"
-           "    -o<output>  Write output to file <output>\n"
+           "    -m<mapfile>  Write a mapfile to <mapfile>\n"
 		   "\n"
-           "    -s<symbol>  Strip unused sections, rooting the section containing <symbol>\n"
-           "                <symbol> is used as entry point when support by output format\n"
+           "    -o<output>   Write output to file <output>\n"
+		   "\n"
+           "    -s<symbol>   Strip unused sections, rooting the section containing <symbol>\n"
+           "                 <symbol> is used as entry point when support by output format\n"
     );
     exit(EXIT_SUCCESS);
 }
@@ -406,7 +416,7 @@ handleMemoryConfigurationOption(const string* target) {
 
 
 static void
-writeOutput(const char* g_outputFilename) {
+writeOutput(void) {
 	switch (g_outputFormat) {
 		case FILE_FORMAT_NONE:
 			error("Output format not specified");
@@ -469,7 +479,7 @@ handleOption(const char* option) {
 
 			g_targetDefined = true;
 			string* filename = str_ToLower(str_Create(&option[1]));
-			mmap_Read(filename);
+			mdef_Read(filename);
 			str_Free(filename);
 			return true;
 		}
@@ -493,6 +503,11 @@ handleOption(const char* option) {
 			str_Free(target);
 			return true;
 		}
+		case 'l':	/* Map file */
+			if (option[1] == 0) error("option \"l\" needs an argument");
+
+			g_listFilename = &option[1];
+			return true;
 		case 'm':	/* Map file */
 			if (option[1] == 0) error("option \"m\" needs an argument");
 
@@ -562,6 +577,10 @@ main(int argc, char* argv[]) {
 
     smart_Process(g_smartlink);
 
+    if (group_NeedsOverlay() && !format_SupportsOverlay(g_outputFormat)) {
+		error("Output format does not support overlay files");
+	}
+
     if (!format_SupportsReloc(g_outputFormat)) {
         assign_Process();
 		sect_ResolveUnresolved();
@@ -573,17 +592,26 @@ main(int argc, char* argv[]) {
         format_SupportsImports(g_outputFormat));
 
 	if (g_outputFilename != NULL) {
-		writeOutput(g_outputFilename);
+		writeOutput();
 	}
 
+	sect_SortSections();
+
     if (g_mapFilename != NULL) {
         if (!format_SupportsReloc(g_outputFormat)) {
-            sect_SortSections();
             map_Write(g_mapFilename);
         } else {
             error("Output format does not support producing a mapfile");
         }
     }
 
+    if (g_listFilename != NULL) {
+        if (!format_SupportsReloc(g_outputFormat)) {
+            list_Write(g_listFilename);
+        } else {
+            error("Output format does not support producing a mapfile");
+        }
+    }
+
     return EXIT_SUCCESS;
 }
